//	Pawe Sikorski
//	Przykad uycia algorytmu z biblioteki stl -
//		stable_partition

//template<class BidirectionalIterator, class Predcate>
//BidirectionalIterator stable_partition(	BidirectionalIterator first,
//											BidirectionalIterator last,
//											Predicate pred);

//stable_partition umieszcza wszystkie elementy podanego zbioru speniajce
//obiekt funkcyjny predykatu przed elementami, ktre go nie speniaj.
//Zachowane zostaje wzgldne uporzdkowanie elementw w kadej grupie.

//Dla porwnania, algorytm partition take umieszcza wszystkie elementy podanego
//zbioru speniajce obiekt funkcyjny predykatu przed elementami, ktre go nie
//speniaj. Jednak w tym przypadku kolejno wewntrzna nie musi by zachowana.

#include <iostream>		//cout
#include <algorithm>	//stable_partition
#include <vector>		//vector
#include <string>		//string
#include <cstdlib>		//rand
#include <ctime>		//time
using namespace std;

//uywane typy wyliczeniowe
typedef enum {BASKETBALL, VOLEYBALL} InterestType;
typedef enum {MAN, WOMAN} SexType;
//Liczba rekordw
#define PEOPLE_NUM	40

//Klasa przedstawiajca osob,
//	mczyzn bd kobiet,
//	interesujc si koszykwk bd siatkwk
//	Dodatkowo kada osoba ma unikalny nr ID
//		(do jednoznacznego jej rozpoznania)
class Person{
private:
	static unsigned ID;
	static const string SexName[];
	static const string InterestName[];
	unsigned _myID;
	InterestType _hobby;
	SexType	_sex;
public:
	Person(InterestType h=BASKETBALL, SexType s=MAN): _myID(ID++),_hobby(h), _sex(s) {}
//	Dla metod Predicate.*
	SexType getSex() const	{return _sex;}
	InterestType getHobby() const	{return _hobby;}

	unsigned getID() const	{return _myID;}
	const string& getHobbyName() const	{return InterestName[_hobby];}
	const string& getSexName() const	{return SexName[_sex];}
//	Wypisz do strumienia
	friend ostream& operator<<(ostream& str,const Person& p){
		return str<<p.getID()<<"\t"<<p.getHobbyName()<<"\t"<<p.getSexName();
	}
};

//Inicializacja statycznych skadowych klasy Person
unsigned Person::ID=1;
const string Person::SexName[]={"Mczyzna","Kobieta"};
const string Person::InterestName[]={"Koszykwka","Siatkwka"};

//Klasa zbierajca funkcje sprawdzajce odpowiednio
//pe oraz zainteresowanie
class Predicate{
public:
	static bool sex(const Person &p){
		return (p.getSex()==WOMAN);
	}

	static bool hobby(const Person &p){
		return (p.getHobby()==BASKETBALL);
	}
};

void Print(vector<Person> & people);//Deklaracja

int main() {
	vector<Person> people;
	srand ( time(NULL) );
	cout<<"--------------------------------------\n"
		<<"Osoby ustawiaj si w kolejk do kas\n"
		<<"Z tym e jest jedna kolejka a dwie kasy\n"
		<<"(z rnymi biletami), a osoby rnej pci\n"
		<<"i o rnych zainteresowaniach przychodz\n"
		<<"losowo.\n";
	for(unsigned i=0;i<PEOPLE_NUM;++i)	//wypenianie kolejki
		people.push_back(Person(InterestType(rand() % 2),SexType(rand() % 2)));
	Print(people);

//	Iteratory, ktre bd wskazyway punkty graniczne zainteresowa
//	i pci w wektorze people;
	vector<Person>::iterator hobby_it,hobby_women_it,hobby_men_it;

//	Sortowanie listy wzgldem hobby (pe bez rnicy)
	hobby_it=stable_partition(people.begin(),people.end(),Predicate::hobby);
	cout<<"\nPo dzieleniu wzgledem hooby\n";
    Print(people);

//  Sortowanie list (czci zainteresowanych koszykwk i siatkwk)
//  wzgldem pci
	hobby_women_it=stable_partition(people.begin(),hobby_it,Predicate::sex);
	hobby_men_it=stable_partition(hobby_it,people.end(),Predicate::sex);
	cout<<"\nPo dzieleniu wzgldem hobby i pci\n";
	Print(people);

//	Dzieki takiemu posortowaniu, zachowana jest kolejnosc przychodzacych ludzi oraz dodatkowo
//	zostali oni podzieleni pomidzy dwa okienka (bilety koszykwka, bilety siatkwka).
//	Co wicej dziki drugiemu sortowaniu nie tylko "wystpuje" podzia pomidzy okienkami,
//	ale take mczyni "przepucili" kobiety na pocztek kolejek.

	return 0;
}

//Funkcja "drukujca" na dany strumie
void Print(vector<Person> & people){
	cout<<"------------------------------\n";
	cout<<"ID\tHOBBY\t\tPEC\n";
    cout<<"------------------------------\n";
    vector<Person>::const_iterator i;
    for(i = people.begin();i != people.end();++i)	cout << (*i) << "\n";
	cout<<"---------------------------------\n";
}
